<?php

/**
 # com_jvframwork - JV Framework
 # @version		1.5.x
 # ------------------------------------------------------------------------
 # author    Open Source Code Solutions Co
 # copyright Copyright (C) 2011 joomlavi.com. All Rights Reserved.
 # @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL or later.
 # Websites: http://www.joomlavi.com
 # Technical Support:  http://www.joomlavi.com/my-tickets.html
-------------------------------------------------------------------------*/

defined('_JEXEC') or die('Restricted access');

class JVAdminLayout extends JVLayout {
    private $_moduleList;
    private $css;
    
    function JVAdminLayout(){
		$args = func_get_args();
		call_user_func_array(array(&$this, '__construct'), $args);
	}    
    
    public function __construct($options) {
        $this->activeTheme = JVFactory::getFramework()->getActiveTheme();
        $this->options = $options;
        $this->getModuleList();
    }
    
    public static function getInstance($options = null){
        static $instance;

		if (empty($instance)) {
            $instance = new JVAdminLayout($options);
		}
        
        return $instance;
    }
    
    /**
     * Parse Layout data to html and joomla jdoc.
     * 
     * @param  array  $mainlayout main layout data.
     * @param  string $layout_type_name 
     * @return string html.
     */
    public function getLayout(){        
        $layouts = $layout = array();
        $reader  = new JVLayoutReader($this, $this->options);   
        $layouts = $reader->read();  
      
        if($layouts == false){
            return false;
        } 
        
        $this->setLayoutData($layouts);
        
        $layouttype = $this->getLayoutType(JVFactory::isRTL());
        $this->options->def('layout_type_name', $layouttype['layouttype']);
        
        if (count($layouttype['layouttypesetting'])) {
            $this->switchSidebar($layouttype['layouttypesetting']);
        }
                    
        if ($this->get('setting') != '') {
            $setting1 = $this->opSettings($this->get('setting'));                     
            $this->applySetting($this->get('setting'));            
            $this->applySetting($setting1);
        }
        

        $html = '<div id="layout">';
        $html .= $this->parser();
        $html .= '</div>';
        
        $this->getStyle($layouttype['layouttype']);
        $document = &JFactory::getDocument();
        $document->addCustomTag("<style type='text/css'>" . $this->css . "</style>");

        if ($html) {
            return $html;
        }
        return;

    }
    
    /**
     * Parse Layout data to html and joomla jdoc.
     * 
     * @access public 
     * @return string html and joomla jdoc.
     */
    public function parser(){ 
        $html = '';
        
        if(is_array($this->get('main'))){
            foreach($this->get('main') as $mainItem){
                if(is_array($mainItem) && isset($mainItem['name'])){
                    
                    if ($mainItem['name'] != 'sidebar1' && $mainItem['name'] != 'sidebar2' && $mainItem['name'] != 'head') {
                        $html .= "<div id='g-{$mainItem['name']}' class='group'>";
                        $html .= "
                        <p props='{}'>
                            <span class='group-handler'>{$mainItem['name']}</span>
                            <span class='toolbox'>
                                <a href='javascript:void(0);' class='icon-add-block' title='add block'>A</a> 
		                        <a href='javascript:void(0);' class='icon-toggle-group' title='collapse/expand'>C/E</a>
                            </span>                                    
                        </p>";                    
                    }
                    
                    if ($this->isSideBar($mainItem['name'])) {
                        $html .= "<div class='{$mainItem['name']}'>";
                        $html .= "<div id='g-{$mainItem['name']}' class='group'>";
                        $html .= "
                        <p props='{}'>
                            <span class='group-handler'>{$mainItem['name']}</span>
                            <span class='toolbox'>
                                <a href='javascript:void(0);' class='icon-add-block' title='add block'>A</a> 
	                         	<a href='javascript:void(0);' class='icon-toggle-group' title='collapse/expand'>C/E</a>
                            </span>                                    
                        </p>";
                    }
                    
                    $group = $this->getGroup($mainItem['name']); // Get group data
                    
                    if(count($group)){
                        foreach($group as $block){
                            if( isset($mainItem['type'])){
                                $block['option']['type'] = $mainItem['type'];
                            }
                            $html .= $this->autoPosition($block, $mainItem['name']);                            
                        }
                    }else{
                        $html .= '<div class="block clearfix block-placeholder">[empty group]</div>';
                    }
                    
                    if ($this->isSideBar($mainItem['name'])) {
                        $html .= "</div>";
                        $html .= "</div>";
                    }  
                    
                    if ($mainItem['name'] != 'sidebar1' && $mainItem['name'] != 'sidebar2' && $mainItem['name'] != 'head') {
                        $html .= '</div>';
                    }     
                                       
                }else{                    
                    $html .= (string ) $mainItem;
                }                
            }
        }
        
        return ($html);
        
    }


    /**
     * Load modules  
     * 
     * @param array $block Block data. 
     * @access public 
     * @return string html. 
     */
    public function autoPosition($block, $groupname){        
        if(isset($block['option']['name']) == false || $groupname == 'head') return false;
        
        $html  = $posclass = '';
        $id    = isset($block['option']['id']) ? $block['option']['id'] : '';
       
        if (isset($block['option']['equal']) && $block['option']['equal'] !='' && $block['option']['equal'] !='false'  && $block['option']['equal'] !='height') {
            $block['option']['class'] = 'modulebox';
            $counter = 0;
            if (isset($block['items']) && is_array($block['items'])) {
                foreach ($block['items'] as $k => $v) {
                    if (isset($v['name'])) {
                        $counter++;
                    }
                }
            }

            if ($counter != 0) {
                $cls = substr($id, -4);
                $name = 'jv-box-' . $cls;
                $posclass = '' . $name . ' box';
                $w = (100 / $counter);
                $this->css .= ".$name {width: $w%;}";
            }
        }
        
        $class = isset($block['option']['class']) ? 'class="'. $block['option']['class']. ' block clearfix"': 'class="block clearfix"';
        
        $prop = '';
        if (count($block['option'])) {
            $prop = json_encode($block['option']);
        }
        
        $html .= "<div name='$groupname' id='b-{$block['option']['name']}' $class>";        
        $html .= "
        <p props='{$prop}'>
            <span class='block-handler'>{$block['option']['name']}</span>
            <span class='toolbox'>
                <a href='javascript:void(0);' class='icon-add-position' title='add position'>A</a> 
				<a href='javascript:void(0);' class='icon-edit-block' title='edit block'>E</a> 
				<a href='javascript:void(0);' class='icon-delete-block' title='delete block'>D</a> 
				<a href='javascript:void(0);' class='icon-toggle-block' title='collapse/expand'>C/E</a>
            </span>                                    
        </p>";        
        if(isset($block['items']) && count($block['items']) > 0){
            foreach($block['items'] as $item){
                if(is_array($item)){
                    $html .= $this->loadModule($item, $block['option']['name'], $posclass);
                }else{
                    $html .= $item;
                }
            }
        }else{
            $html .= '<div class="position position-placeholder">[empty block]</div>'; 
        }
        
        $html .= "</div>\n";

        return $html;
    }

    /**
     * Load module position
     * 
     * @param  array $position position data
     * @access public 
     * @return string html.    
     */
    public function loadModule($position, $blockname, $posclass){
        $position_name = $position['name'];
        $class         = isset($position['class']) ? " ".$position['class']: '';

        if (!in_array($position_name, $this->hiddenposition)) {
            $html = '';
            if ($position_name != '' && $position_name != 'head') {                
                $html .= "<div name='$blockname' id='p-$position_name' class='position{$class} $posclass'>";
                
                $modules = $this->getModuleContent($position_name);
                $class ='';
                if($modules  == array()){
                    $class = 'no-content'; 
                }
                
                $prop = (string )json_encode($position);
                
                $html .= "
                 <p props='{$prop}'>
                    <span class='position-handler'>{$position_name}</span>
                    <span class='toolbox'>
                        <a href='javascript:void(0);' class='icon-edit-position' title='edit position'>E</a> 
						<a href='javascript:void(0);' class='icon-delete-position' title='delete position'>D</a>
                    </span>                                    
                </p>";  
                
                $html .= $this->demoModule($position_name);
                $html .= "</div>";
            }
            return $html;
        }
        return;
    }

    /**
     * @return Custom style
     */
    function getStyle($layout = 'default'){ 
        $layout = $this->options->getParam('layouttype', 'left-main-right');
        $templatewidth = 85; // %
        $sidebar1 = 35;
        $sidebar2 = 25;
        $mainbody = 75;
        $main = 65;

        $space = $this->options->getSpaceWidth();
        $fontsize = $this->options->getFontSize();
        $fontfamily = $this->options->getParam('fontfamily', '');
        $templatecolor = $this->options->getTemplateColor();
        $templatehovercolor = $this->options->getTemplateHoverColor();
        $this->css .= "#layout, #layout-tool {width: " . ($templatewidth) . "%; margin: 0 auto}";
        $this->css .= "." . $this->getSideBar2() . "{width:{$sidebar2}%; float: left}";
        $this->css .= "." . $this->getSideBar1() . "{width:{$sidebar1}%; float: right}";
        $this->css .= ".jv-inner{position:relative;}";
        $this->css .= ".jv-inner{position:relative;}";
        $this->css .= "#maincontent{width:{$main}%;}";
        $this->css .= "#jv-mainbody{float:left; width: {$mainbody}%}";

        switch ($layout) {
            case 'right-main-left':
				$this->css .= "#g-" . $this->getSideBar2() . "{margin:0 {$space}px 0 0;}";
		        $this->css .= "#g-" . $this->getSideBar1() . "{margin:0 0 0 {$space}px;}";
            
			    $this->css .= "#jv-mainbody {float: right}";
                $this->css .= "#maincontent{float: left}";;
                break;

            case 'main-left-right':
            case 'main-right-left':
                $this->css .= "#maincontent{float: left}";
				$this->css .= "#g-" . $this->getSideBar1() . "{margin:0 0 0 {$space}px;}";	
				$this->css .= "#g-" . $this->getSideBar2() . "{margin:0 0 0 {$space}px;}";	
                break;

            case 'left-right-main':
            case 'right-left-main':
				$this->css .= "#g-" . $this->getSideBar2() . "{margin:0 {$space}px 0 0;}";
				$this->css .= "#g-" . $this->getSideBar1() . "{margin:0 {$space}px 0 0;}";
                $this->css .= "#jv-mainbody {float: right !important}";
                break;

            case 'main-right':
				$this->css .= "#g-" . $this->getSideBar2() . "{margin:0 0 0 {$space}px;}";
                $this->css .= "#g-" . $this->getSideBar1() . "{ display: none}";
                break;

            case 'right-main':
				$this->css .= "#g-" . $this->getSideBar2() . "{margin:0 {$space}px 0 0;}";
                $this->css .= "#jv-mainbody {float: right !important}";
                $this->css .= "#g-" . $this->getSideBar1() . "{ display: none}";;
                break;

            case 'default':
            case 'left-main-right':
				$this->css .= "#g-" . $this->getSideBar2() . "{margin:0 0 0 {$space}px;}";
		        $this->css .= "#g-" . $this->getSideBar1() . "{margin:0 {$space}px 0 0 ;}";
                break;

            case 'left-main':
                $this->css .= "#g-" . $this->getSideBar2() . "{ display: none}";
				$this->css .= "#g-" . $this->getSideBar1() . "{margin:0 {$space}px 0 0 ;}";
                break;

            case 'main-left':
				$this->css .= "#g-" . $this->getSideBar2() . "{margin:0 0 0 {$space}px;}";
                $this->css .= "#g-" . $this->getSideBar1() . "{ display: none}";
                break;
			 case 'main':
				$this->css .= "#g-" . $this->getSideBar1() . "{ display: none}";
				$this->css .= "#g-" . $this->getSideBar2() . "{ display: none}";
				break;

            default:
                if ($this->layout[$this->getSideBar1()] == $this->getSideBar2()) {
                }
                if ($this->layout[$this->getSideBar2()] == $this->getSideBar1()) {
                }
                break;
        }
    }

    /**
     * Module demo 
     *
     * @return string hml
     */
    function demoModule($name)
    {
        $html  = '';
        $html .= "<div class='module'>";
        if($name=='component'){
             $html .= "<p>".$name."</p>";
        } else{
            $modules = $this->getModuleContent($name);
            $class = '';
            if($modules == array() && $name !='component'){
                $class = 'no-content';
            }
            
            if($this->options->getLogoPosition() == $name){
                $html .= "<p>Your Logo</p>";    
            }
            
            if($modules != array() || $name=='component'){
                foreach($modules as $module){
                    $title = $module->title;
                    if($name == 'component'){
                        $title = 'component';
                    }
                    $html .= "<p>".$module->title."</p>";
                }
            } else{
                $html .= "<p>no module published !!</p>"; 
            }
        }
        
        $html .= "</div>";

        return $html;
    }
    
    function getModuleList(){
        if($this->_moduleList == null){
            $db =& JFactory::getDBO();       
            //Get default template
            $query = 'SELECT position, title FROM #__modules WHERE client_id=0 && published=1';
            $db->setQuery($query);
            $module = $db->loadObjectList();
           
            $this->_moduleList = $module;
        }
        return $this->_moduleList;
    }
    
    function getModuleContent($pos){
        $modules = array();
        $moduleList = $this->_moduleList;
        foreach($moduleList as $module){
            if($module->position == $pos){
                $modules[] = $module;
            }
        }        
        return $modules;
    }
    
    public function hasPosition($blockname){
        $block  = $this->getBlock($blockname);
        $haspos = false;
        if(count($block['items'])){
            foreach($block['items'] as $item){
                if(isset($item['name'])){
                    $haspos = true;
                    
                }
            }
        }
        
        return $haspos;
    }
}