<?php
/**
 # com_jvframework - JV Framework
 # @version		1.5.x
 # ------------------------------------------------------------------------
 # author    Open Source Code Solutions Co
 # copyright Copyright (C) 2011 joomlavi.com. All Rights Reserved.
 # @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL or later.
 # Websites: http://www.joomlavi.com
 # Technical Support:  http://www.joomlavi.com/my-tickets.html
-------------------------------------------------------------------------*/

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

class JVLayout extends JObject{
    protected $options = array();
    protected $main = array();
    protected $data = array();    
    protected $setting = array();
    protected $hiddenblock = array();
    protected $hiddenposition = array();
    protected $activeTheme;
    protected $sidebar1 = 'sidebar1';
    protected $sidebar2 = 'sidebar2';  
    
    function JVLayout(){
		$args = func_get_args();
		call_user_func_array(array(&$this, '__construct'), $args);
	}    
    
    public function __construct($options) {
        $this->activeTheme = JVFactory::getFramework()->getActiveTheme();
        $this->options = $options;
    }
    
    public static function getInstance($options = null){
        static $instance;

		if (empty($instance)) {
            $instance = new JVLayout($options);
		}
        
        return $instance;
    }
    
    public function getLayout(){
        $layouts = $layout = array();
        $reader  = new JVLayoutReader($this, $this->options);
        $cache   = JVHooks::getCache();
        
        if(JVFactory::getUserAgent() != 'standard'){
            $layouts = $reader->read();
        }else{
            $layouts = $cache->call(array($reader, 'read'), '', 'string');
        }
        
        if($layouts == false){
            return false;
        }   
        
        $this->setLayoutData($layouts);
        
        $hook = JVHooks::getInstance($this, $this->options); 
        $hook->fromDocument(); //Get head data from document;
        
        $layouttype = $this->getLayoutType(JVFactory::isRTL());
        
        $this->options->def('layout_type_name', $layouttype['layouttype']);
                 
        if (count($layouttype['layouttypesetting'])) {
            $this->switchSidebar($layouttype['layouttypesetting']);
        };        
                    
        if ($this->get('setting') != '') {
            $setting1 = $this->opSettings($this->get('setting'));                     
            $this->applySetting($this->get('setting'));            
            $this->applySetting($setting1);
        }
              
        $parser = new JVLayoutParser($this, $this->options);
        $layout = $parser->parse(); 
                        
        $hook->putHead();
        return $layout;
        
    }  
    
    /**
     * Check current layout type setting and return layout type.
     *          
     * @access public   
     * @param  $direction   
     * @return array layout type and layout type setting.
     */
    public function getLayoutType($isRTL) {
        $params = JVFactory::getParameter();        
        $layouttypesetting = array();
        $layouttype = 'left-main-right';
        $layouttype = $params->getLayoutType();
        
        if($layouttype == 'default'){
            $layouttype = 'left-main-right';
        }
        
        switch ($layouttype) {
            case 'left-main-right':
            case 'default':
                if ($isRTL) {
                    $layouttype = 'right-main-left';
                }
                break;

            case 'left-main':
                $layouttypesetting = array($this->getSideBar2() => 'hidden');
                if ($isRTL) {
                    $layouttypesetting = array($this->getSideBar1() => 'hidden', $this->getSideBar2
                        () => $this->getSideBar1());
                    $layouttype = 'main-left';
                }
                break;

            case 'main-left':
                $layouttypesetting = array($this->getSideBar1() => 'hidden', $this->getSideBar2
                    () => $this->getSideBar1());
                if ($isRTL) {
                    $layouttypesetting = array($this->getSideBar2() => 'hidden');
                    $layouttype = 'left-main';
                }

                break;

            case 'left-right-main':
                $layouttypesetting = array($this->getSideBar2() => $this->getSideBar1(),
                    $this->getSideBar1() => $this->getSideBar2());
                if ($isRTL) {
                    $layouttype = 'main-right-left';
                }

                break;

            case 'right-main-left':
                if ($isRTL) {
                    $layouttype = 'left-main-right';
                }
                break;

            case 'right-left-main':
                if ($isRTL) {
                    $layouttype = 'main-left-right';
                }
                break;

            case 'right-main':
                $layouttypesetting = array($this->getSideBar1() => 'hidden');
                if ($isRTL) {
                    $layouttype = 'main-right';
                }
                break;

            case 'main-left-right':

                if ($isRTL) {
                    $layouttype = 'right-left-main';
                }
                break;

            case 'main-right-left':
                $layouttypesetting = array($this->getSideBar2() => $this->getSideBar1(),
                    $this->getSideBar1() => $this->getSideBar2());
                if ($isRTL) {
                    $layouttype = 'left-right-main';
                }
                break;

            case 'main':
                $layouttypesetting = array($this->getSideBar1() => 'hidden', $this->getSideBar2
                    () => 'hidden');
                break;

            case 'main-right':
                $layouttypesetting = array($this->getSideBar1() => 'hidden');
                if ($isRTL) {
                    $layouttype = 'right-main';
                }
                break;
        }

        return array('layouttypesetting' => $layouttypesetting, 'layouttype' => $layouttype);
    }
    
    /**
     * Switch Sidebar
     * 
     * @param  array $layouttypesetting setting of current layout type
     * @access public     
     * @return void
     */
    public function switchSidebar($layouttypesetting) {
        $groups = $this->data;
        if (count($layouttypesetting)) {
            foreach ($layouttypesetting as $k => $v) {
                if ($this->getGroup($k)) {
                    if ($v == 'hidden') {
                        $groupitems = array();
                    } else {
                        $groupitems = $groups[$v];
                    }
                    $this->setGroup($k, $groupitems);
                }
            }
        }
    }
    
    /**
     * opSetting
     * 
     * @param  array $setting layout setting array.
     * @access public     
     * @return void
     */
     protected function opSettings($settings){
        $set = array();        
        foreach($settings as $index => $setting){
            if(isset($setting['items'])){
                foreach($setting['items'] as $key => $item){
                    if(is_array($item) && isset($item['to'])){
                        $to = $item['to'];   
                        unset($item['to']);               
                        $set[$to]['items'][$key] = $item;
                    }                    
                }
                $set[$index]['option'] = $setting['option'];
            }
        }
        return $set;
    }  
    
    
    /**
     * Apply User Setting
     * 
     * @param  array $layoutsetting layout setting array.
     * @access public     
     * @return void
     */
    public function applySetting($settings) {        
        $hiddenposition = $hiddenblock = array();
        if (count($settings)) {
            
            foreach ($settings as $blockname => $setting) {                
                $from   = isset($setting['option']['from'])  ? $setting['option']['from']  : null;
                $to     = isset($setting['option']['to'])    ? $setting['option']['to']    : null;
                $where  = isset($setting['option']['where']) ? $setting['option']['where'] : null; 
                $blocks = isset($setting['items'])           ? $setting['items']           : null;

                if (isset($setting['option']['hidden'])) { // If exist option hidden.
                    $hiddenblock[] = $blockname; // do add blockname to array $hiddenblock.
                    $this->removeGroupItem($from, $blockname);
                } else {
                    /***********************************************************************************/

                    if ($this->getBlock($blockname) && isset($setting['option']) && count($setting['option'])) {
                        $this->setBlockOption($blockname, $setting['option']);
                    }

                    /***********************************************************************************/
                    
                    if ($from) {
                        if ($to) { // move block to other group
                            $this->addGroupItem($to, $this->getBlock($blockname), $where);
                            $this->removeGroupItem($from, $blockname);
                            
                        } else {
                            $this->addGroupItem($from, $setting, $where);
                        }
                    }
                    
                    /***********************************************************************************/
                    if ($blocks) { // if exist setting for block.
                        foreach ($blocks as $posname => $posval) { // Apply setting for block
                            $toblock = isset($posval['to']) ? $posval['to'] : false;
                            $block = $this->getBlock($blockname);
                            if (isset($posval['hidden'])) { // If exist option hidden.
                                $this->removeBlockItem($blockname, $posname); // add position name to array $hiddenposition.
                            } else {
                                if (isset($block['items'][$posname]) && count($block['items'][$posname]) && count($posval)) {
                                    $this->setPositionOption($blockname, $posname, $posval);
                                }
                                if (isset($posval['hidden']) == false) {
                                    $posval['where'] = isset($posval['where']) ? $posval['where'] : null;
                                    if ($toblock) {
                                        $this->addBlockItem($toblock, $posval, $posval['where']);
                                        $this->removeBlockItem($blockname, $posname);
                                    } else {                                       
                                        $this->addBlockItem($blockname, $posval, $posval['where']);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        
        $this->hiddenblock    = $hiddenblock;
        $this->hiddenposition = $hiddenposition;
    }
    
    protected function setLayoutData($layouts){
        foreach($layouts as $key => $layout){
            $this->set($key, $layout);
        }
    }
    
    /**
     * Get All Position In Group 
     * 
     * @param  $groupname
     * @return array position name. 
     */
    public function getGroupPos($groupname) {
        $pos = array();
        $group = $this->getGroup($groupname);
        
        if ($group) {
            foreach ($group as $key => $val) {
                if (!$this->isHiddenBlock($key)) {
                    $block = $this->getBlock($key);
                    if (count($block['items'])) {
                        foreach ($block['items'] as $key => $val) {
                            if (isset($val['name']) && !$this->isHiddenPosition($val['name'])) {
                                $pos[] = $val['name'];
                            }
                        }
                    }
                }
            }
        } else {
            return;
        }
        if (count($pos)) {
            return $pos;
        }
        return;
    }

    /**
     * Get group item data
     * 
     * @param  string $groupname. 
     * @access public
     * @return array group item data.
     */
    public function getGroup($groupname) {
        if(isset($this->data[$groupname])){
            return $this->data[$groupname];    
        }
        
        return array();
    }

    /**
     * Set group item data. 
     * 
     * @param  string $groupname
     * @param  array  $groupitem group item data.
     * @access public 
     * @return void
     */
    public function setGroup($groupname, $groupitem) {
        $this->data[$groupname] = $groupitem;
    }

    /**
     * Check available block in group 
     * 
     * @param  string $groupname
     * @param  string $blockname
     * @access public 
     * @return bool true if available else false 
     */
    public function inGroup($groupname, $blockname) {
        $group = $this->data[$groupname];
        return isset($group[$blockname]);
    }

    /**
     * Add block to group
     * 
     * @param  string $groupname name of the group that you want to add block 
     * @param  array  $block     block data.
     * @param  string $where     new block will be added after $where
     * @access public 
     * @return void
     */
    public function addGroupItem($groupname, $block, $where = null) {
        if(!isset($block['option']['name'])) return false;
         
        $blockname = $block['option']['name'];
        
        if ($this->inGroup($groupname, $blockname)) {            
            if(isset($block['option'])){
                $this->setBlockOption($blockname, $block['option']);
            }
            
            if(isset($block['items'])){
                //$this->setBlockItems($blockname , $block['items']);    
            }
            
            $block = $this->getBlock($blockname);            
            $this->removeGroupItem($groupname, $blockname); 
        }

        $group = $this->getGroup($groupname);
        
        if (is_array($group)) {
            if (isset($where)) {
                if ($where == '') { //add block to top of group
                    $group = array_merge(array($blockname => $block), $group);
                } else { //add block to specific of group
                    if (count($group)) {
                        foreach ($group as $k => $item) {
                            $temparray[$k] = $item;
                            if ($item['option']['name'] == $where) {
                                $temparray[$blockname] = $block;
                            }
                        }

                        $group = $temparray;
                    }
                }
            } else { //add block to bottom of group
                $group = array_merge($group, array($blockname => $block)); 
            }
        }
        
        $this->setGroup($groupname, $group);
        
        return true;
    }

    /**
     * Remove Block From Group
     * 
     * @param  string $groupname the group where yo want to remove block. 
     * @param  string $blockname the name of the block to be removed.
     * @access public 
     * @return void
     */
    public function removeGroupItem($groupname, $blockname) {        
        if (isset($this->data[$groupname][$blockname])) {
            unset($this->data[$groupname][$blockname]);
        }
    }
    
    /**
     *  Create new Block
     * 
     * @param  string $name
     * @param  string $option block option;
     * @param  array  $items
     * @param  string $group  group name
     * 
     * @access public 
     * @return array  block data
     */
    public function newBlock($name, $option = array(), $items = array(), $group = null, $where = null) {
        $block = array();
                
        $option['name']  = $name != '' ? $name : rand(100, 911);   
        $block['option'] = $option;    
        $block['items']  = is_array($items)  ? $items : array();    
            
        if($group != null){
            $this->addGroupItem($group, $block, $where);
        }else{
            return $block;
        }
    }

    /**
     *  Get block data 
     * 
     * @param  string $blockname
     * @access public 
     * @return array  block data, false if block is not available.
     */
    public function getBlock($blockname) {
        foreach($this->data as $group){
            if(count($group))
                foreach($group as $name => $blockdata){
                    if($name == $blockname){
                        return $blockdata;
                    }
                }
        }
        
        return false;
    }

    /**
     * Set block item data
     * 
     * @param  string $blockname
     * @param  array  $blockitems
     * @access public 
     * @return void
     */
    public function setBlockItems($blockname, $blockitems) {
        foreach($this->data as &$group){
            foreach($group as $name => &$blockdata){
                if($name == $blockname){
                    $blockdata['items'] = $blockitems;
                }
            }
        }
    }

    /**
     * Set Block Option
     * 
     * @param  string $blockname
     * @param  array  $option
     * @access public 
     * @return void
     */
    public function setBlockOption($blockname, $option) {
        $block = $this->getBlock($blockname);

        if (count($option)) {
            foreach ($option as $key => $op) {
                if (!in_array($key, array('from', 'where', 'to', 'hidden'))) { // specific option
                    $block['option'][$key] = $op;
                }
            }
        }
        
        foreach($this->data as &$group){
            foreach($group as $name => &$blockdata){
                if($name == $blockname){
                    $blockdata['option'] = $block['option'];
                }
            }
        }
    }

    /**
     * Check available position in block 
     * 
     * @param  string $blockname
     * @param  string $positionname
     * @access public 
     * @return bool true if the position is available, and otherwise.
     */
    public function inBlock($blockname, $positionname) {
        $block = $this->getBlock($blockname);
        return isset($block['items'][$positionname]);
    }
    
    /**
     *  Get enable position in block
     * 
     * @param  string $blockname name of the block that you want to check
     * @access public 
     * @return array  position name.
     */
    public function enablePos($blockname) {
        $block = $this->getBlock($blockname);
        $pos   = array();
        
        if (isset($block['items']) && is_array($block['items'])) {
            foreach ($block['items'] as $key => $val) {
                if (isset($val['name']) && !$this->isHiddenPosition($val['name'])) {
                    if ($this->JFModules($val['name']))
                        $pos[] = $val['name'];
                }
            }
            return $pos;
        }
        return false;
    }

    /**
     * Is block hidden
     * 
     * @param  string $blockname
     * @access public  
     * @return bool true if this block is hidden or false if the block isn't hidden.
     */
    public function isHiddenBlock($blockname) {
        return in_array($blockname, $this->hiddenblock);
    }

    /**
     * Add position to block
     * 
     * @param  string $blockname
     * @param  array  $position  position data
     * @param  string $where     new position will be added after $where
     * @access public 
     * @return void
     */
    public function addBlockItem($blockname, $position, $where = null) {
        $blockitems = array();

        if (!count($position))
            return;

        $positionname = $position['name'];

        if ($this->inBlock($blockname, $positionname)) {
            $this->setPositionOption($blockname, $positionname, $position);
            $position = $this->getPosition($blockname, $positionname);
            $this->removeBlockItem($blockname, $positionname); // Remove position.
        }

        if (isset($where) && !$this->inBlock($blockname, $where)) {
            $where = '';
        }
        $block = $this->getBlock($blockname);

        if (isset($block['items'])) {
            $blockitems = $block['items'];
        }

        if (isset($where)) {
            if ($where == '') { //add position to top of block
                //$blockitems = array($positionname => $position) + $blockitems;
                $blockitems = array_merge(array($positionname => $position), $blockitems);
                
            } else { //add to specific position
                if (count($block)) {
                    foreach ($blockitems as $key => $pos) {
                        $temparray[$key] = $pos;
                        if ($where == $key) {
                            $temparray[$positionname] = $position;
                        }
                    }
                }

                $blockitems = $temparray;
            }

        } else { //add position to bottom of block

            $blockitems = array_merge($blockitems, array($positionname => $position));
        }
        
        $this->setBlockItems($blockname, $blockitems);
    }

    /**
     * Remove position from block
     * 
     * @param  string $blockname    the block where you want to remove position.
     * @param  string $positionname the position to be removed.
     * @access public 
     * @return void
     */
    public function removeBlockItem($blockname, $positionname) {
        foreach($this->data as &$group){
            foreach($group as $name => &$blockdata){                
                if($name == $blockname && is_array($blockdata['items'])){
                    foreach($blockdata['items'] as $index => $position){
                        if($position['name'] == $positionname){
                            unset($blockdata['items'][$index]);
                        }
                    }
                }
            }
        }
        return true;
    }

    /**
     * Get position data
     * 
     * @param  string $blockname
     * @param  string $positionname
     * @access public 
     * @return array position data
     * 
     */
    public function getPosition($blockname, $positionname) {
        $block = $this->getBlock($blockname);
        if (isset($block['items'][$positionname])) {
            return $block['items'][$positionname];
        }
        return false;
    }

    /**
     * Set Position Option
     * 
     * @param  string $blockname
     * @param  string $positionname     name of position to be set
     * @param  array  $possition_option all option. 
     * @access public 
     * @return void
     */
    private function setPositionOption($blockname, $positionname, $possition_option) {
        $position = $this->getPosition($blockname, $positionname);

        if (count($possition_option)) {
            foreach ($possition_option as $key => $op) {
                if (!in_array($key, array('from', 'where', 'to', 'hidden'))) { // specific option
                    $position[$key] = $op;
                }
            }
        }
        
        foreach($this->data as &$group){
            foreach($group as $name => &$blockdata){
                if($name == $blockname && is_array($blockdata['items'])){
                    foreach($blockdata['items'] as $index => $item){
                        if($item['name'] == $positionname){
                            $blockdata['items'][$index] = $position;
                        }
                    }
                }
            }
        }
    }

     /**
     * Is position hidden
     * 
     * @param  string $blockname
     * @access public  
     * @return bool true if this position is hidden and otherwise.
     */
    public function isHiddenPosition($positionname) {
        if (in_array($positionname, $this->hiddenposition))
            return true;
        return false;
    }

    /**
     * Check available enable module in this group.
     * 
     * @param  string $groupname group
     * @access public 
     * @return bool true if there is at least 1 enabled module else false.
     */
    public function hasGroupItems($groupname) {
        $groupPos = $this->getGroupPos($groupname);
                
        if (count($groupPos)) {
            $pos = implode(' or ', $groupPos);
            
            if ($this->JFModules($pos) && $this->getGroup($groupname)) {
                return true;
            }
        }
        return false;
    }    
    
    /**
     * Set sidebar1
     * 
     * @param  string $groupname
     * @access public  
     * @return void
     */
    public function setSideBar1($groupname) {
        $this->sidebar1 = $groupname;
    }

    /**
     * Get sideBar1
     * 
     * @access public  
     * @return string groups name
     */
    public function getSideBar1() {
        return $this->sidebar1;
    }


    /**
     * Set sidebar2
     * 
     * @param  string $groupname
     * @access public  
     * @return void
     */
    function setSideBar2($groupname) {
        $this->sidebar2 = $groupname;
    }

    /**
     * Get sideBar2
     * 
     * @access public  
     * @return string groups name
     */
    public function getSideBar2() {
        return $this->sidebar2;
    }

    /**
     * Check if groups is sideBar
     * 
     * @param  string $groupname
     * @access public  
     * @return bool true if this group is sidebar and otherwise.
     */
    public function  isSideBar($groupname) {
        if ($groupname == $this->sidebar1 || $groupname == $this->sidebar2)
            return true;
        return false;
    }
        
    /**
     * Count module in postion.
     * 
     * @param string $modules
     * @access public
     * @return int the number of modules enabled in one or more module positions.
     */
    public function JFModules($modules) {
        $doc = JVFactory::getFramework();
        return $doc->getDocument()->countModules($modules);
    }
    
    /**
     * Get template name
     * 
     * @access public   
     * @return string templatename.
     */
    public function  getTemplate() {
        return JVFactory::getFramework()->getDocument()->template;
    }

    /**
     * Template url
     * 
     * @access public    
     * @return string link to template.
     */
    public function  getTmplurl() {
        return JURI::root(). 'templates/' . $this->getTemplate();
    }

    /**
     * Theme url
     * 
     * @access public    
     * @return string link to current themes.
     */
    public function  getThemesUrl() {
        return $this->getTmplurl() . '/themes/' . $this->activeTheme->theme;
    }
    
    /**
     * Base theme url
     * 
     * @access public    
     * @return string link to base themes.
     */
    public function getBaseThemesUrl() {
        return JURI::base().'components/com_jvframework/basethemes';
    }

    /**
     * Template path
     * 
     * @access public   
     * @return string template path
     */
    public function templatePath() {
        return JVPATH_TEMPLATE . $this->getTemplate() . DS;

    }
    
    /**
     * Layout path
     * 
     * @return string path to current layout 
     */
    public function layoutPath() {
        return $this->templatePath() . 'themes' . DS . $this->activeTheme->theme;
    }
   
    /**
     * Check if Themes is Available
     * 
     * @access public 
     * @return bool true if theme available else false.
     */
    public function themesAvailable() {
        if (file_exists($this->templatePath() . 'themes' . DS . JVParam::layoutOverrideDir() . DS)) {
            return true;
        }
        return false;
    }

    /**
     * Layout override path.
     * 
     * @access public
     * @return string override path.
     */
    public function overridePath() {
        return $this->templatePath() . 'themes' . DS . $this->activeTheme->theme . DS . 'layouts' . DS;
    }

    /**
     * Mobile layout override path.
     * 
     * @access public
     * @return string override path.
     */
    public function mobileOverridePath() {
        return $this->templatePath() . 'themes' . DS . $this->activeTheme->theme . DS . 'mobiles' . DS;
    }

}