<?php
/**
 # com_jvframework - JV Framework
 # @version		1.5.x
 # ------------------------------------------------------------------------
 # author    Open Source Code Solutions Co
 # copyright Copyright (C) 2011 joomlavi.com. All Rights Reserved.
 # @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL or later.
 # Websites: http://www.joomlavi.com
 # Technical Support:  http://www.joomlavi.com/my-tickets.html
-------------------------------------------------------------------------*/

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');        
        
class JVLayoutReader{
    protected $options;
    protected $layouts;
    
	function JVLayoutReader(){
		$args = func_get_args();
		call_user_func_array(array(&$this, '__construct'), $args);
	}
    
    public function __construct($layouts, $options) {
        $this->layouts = $layouts;
        $this->options = $options;
    }
    
    public function read(){
        $layouts = $data = array();
        $layoutSetting = $this->getLayoutSetting();
        
        try{
           $mainLayout = $this->getMainLayout();            
        }catch(Exception $e){
            echo 'Message: ' .$e->getMessage();
            return false;
        }
        
        if(is_array($mainLayout)){
            foreach ($mainLayout as &$layout){
                if(is_array($layout) && isset($layout['name'])){
                    try{
                        $groupLayout  = $this->getGroupLayout($layout['name']);                        
                        if(is_array($groupLayout)){                            
                            $data[$layout['name']] = $groupLayout;
                        }
                        
                    }catch(Exception $e){
                        echo 'Message: ' .$e->getMessage();
                    }  
                }        
            }            
        }

        if(count($mainLayout) && count($data)){
            return array('main'=>$mainLayout, 'data'=>$data, 'setting'=>$layoutSetting);
        }
         
        return;
    }
    
    private function getLayoutSetting(){
        $layoutSetting = JVLayoutReader::toArray($this->getXmlObject($this->options->def('layoutsetting', ''), false));
        
        if(is_array($layoutSetting)){
            return $layoutSetting;
        }
        
        return;
    }
    
    private function getMainLayout(){        
        $layout = array();        
        $mobilelayout = $this->options->getParam(JVFactory::getUserAgent().'_layout', -1) != -1 ? $this->options->getParam(JVFactory::getUserAgent().'_layout', -1) : $this->options->getParam('handheld_layout', 'handheld');
        
        if (JVFactory::getUserAgent() == 'standard') {
            $mainPath = JVPATH_BASELAYOUT.'main.xml';
            $theme    = $this->layouts->overridePath() .'main.xml';
            if(is_file($theme)){
                $mainPath = $theme;
            }
            
        } else {
            $mainPath = JVPATH_BASELAYOUT_MOBILE. $mobilelayout . DS .'main.xml';            
            $theme    = $this->layouts->mobileOverridePath() . $mobilelayout . DS .'main.xml';
            
            if(is_file($theme)){
                $mainPath = $theme;
            }
        }
        
        if(!file_exists($mainPath)){
            throw new Exception("Main layout not found !<br />");
        }
        
        $mainLayout = JVLayoutReader::toArray($this->getXmlObject($mainPath));
        
        if(is_array($mainLayout)){
            return $mainLayout;
        }
        
        return;
    } 
    
    private function getGroupLayout($layout){
        $layouts  = array();
        $useragent= JVFactory::getUserAgent();
        $base     = JVPATH_BASELAYOUT . $layout . '.xml';
        $theme    = $this->layouts->overridePath() . $layout . '.xml';
        $mobilelayout = $this->options->getParam(JVFactory::getUserAgent().'_layout', -1) != -1 ? $this->options->getParam(JVFactory::getUserAgent().'_layout', -1) : $this->options->getParam('handheld_layout', 'handheld');
            
        if ($useragent == 'standard') {            
            if(file_exists($theme)){
                $path = $theme;  
            }else{
                $path =  $base;  
            }
            
        } else {
            $handheld= JVPATH_BASELAYOUT_MOBILE . 'handheld' . DS .$layout. '.xml';
            $basemb  = JVPATH_BASELAYOUT_MOBILE .$mobilelayout. DS . $layout . '.xml';
			$themehd = $this->layouts->mobileOverridePath() . 'handheld' . DS .$layout. '.xml';
            $thememb = $this->layouts->mobileOverridePath() . $mobilelayout .DS. $layout . '.xml';            
            
            if(JFile::exists($thememb)){
                $path = $thememb;
				
            } elseif(JFile::exists($basemb)){
                $path = $basemb;
				
            }elseif(JFile::exists($themehd)){
                $path = $themehd;
            } 
			elseif(JFile::exists($handheld)){
                $path = $handheld;
				
            } else{
                $path = $base;
				
            }
        }
               
        if(!file_exists($path)){
            throw new Exception($layout." not found !<br />"); return false;
        }
        
        $layouts = JVLayoutReader::toArray($this->getXmlObject($path));
        
        return $layouts;
    } 
    
    function toArray($xml){
        $array = array();
        
        if(is_object($xml)){
               
            if(in_array($xml->name(), JVLayoutReader::getLayoutTag()) == false){
                if($xml->name() != 'layout'){
                    $array = array_merge($array, array(self::startTag($xml->name(), $xml->attributes(), $xml->data())));    
                }                                
            }else{
                $xml->addAttribute('tag', $xml->name());
                $xml->addAttribute('data', trim($xml->data()));
                
                if($xml->name() == 'tmpls'){
                    $array[$xml->attributes('name')]['option'] = (array) $xml->attributes(); 
                }elseif( $xml->name() == 'tmpl'){
                    $array[$xml->attributes('name')] =  (array) $xml->attributes();
                }else{  
                    
                    $array = array($xml->attributes());
                }                
            }
                       
            if (count($xml->children())):
                foreach ($xml->children() as $child) {                    
                    if($xml->name() == 'tmpls'){
                        if(isset($array[$xml->attributes('name')]['items'])){
                            $array[$xml->attributes('name')]['items'] = array_merge((array) $array[$xml->attributes('name')]['items'], JVLayoutReader::toArray($child));    
                        }else{
                            $array[$xml->attributes('name')]['items'] = JVLayoutReader::toArray($child);
                        }
                         
                    }else{
                        $array = array_merge($array, JVLayoutReader::toArray($child));
                    }
                    
                }
                
            endif;
            
            if(in_array($xml->name(), JVLayoutReader::getLayoutTag()) == false){
                if($xml->name() != 'layout' && in_array($xml->name(), self::uncloseTag()) != true){
                    $array = array_merge($array, array(self::closeTag($xml->name())));    
                }
            }
            
        }
        
        return $array;
    }    
   
    private function getXmlObject($xml, $file = true) {
        $xmlobj = &JFactory::getXMLParser('Simple');
        
        if ($file) {
            $xmlobj->loadFile($xml);
        } else {
            $xmlobj->loadString($xml);
        }
        
        return $xmlobj->document;
    }
    
    
    private function startTag($tag, $atts, $data){
        $attribute = '';
        
        if(count($atts)){
            foreach ($atts as $k => $v) {
                $attribute .= " $k='$v'";
            }            
        } 
        
        $slash = in_array($tag, self::uncloseTag()) ? '/' : '';   
            
        return "<{$tag}{$attribute}{$slash}>".trim($data);
    }
    
    private function closeTag($tag){
        return '</'.$tag.'>';
    }
    
    private function uncloseTag(){
        return array('input', 'img', 'br', 'hr');
    }    
    
    private function getLayoutTag(){
        return array('tmplgroup','tmpls','tmpl','menu','gfont','scriptfile','stylefile','style','addonstyle','extensionstyle','customheadtag');
    }
    
}